package com.blazinggames.cq1;

import java.awt.*;
import java.awt.image.*;

/**
 * Coffee Quest Open Source Release
 * Copyright (C) 1997-1998, 2002,2004 Blazing Games Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ---------------------------------------------------------------
 * 
 * A display aid to help in rendering the 3D display. As the game uses
 * the painters algorithm, walls are created using multiple layers or "cells".
 * 
 * @author Billy D. Spelchan
 * @version 2.10.040103
 */
class ImgCell
//extends Canvas
{
	static final int MZ_DispX[] = {
		0,41,100,139,189, 0,0,65,159,218, 0,20,194, 0,239};
	static final int MZ_DispY[] = {
		77,77,77,77,77, 58,50,50,50,58, 15,15,15, 0,0};
//	static final int MZ_Width[] = {
//		71,80,60,80,71, 42,100,130,100,42, 66,220,66, 21,21};
	static final int MZ_Width[] = {
		71,80,60,80,71, 42,100,132,100,42, 66,220,66, 21,21};
	static final int MZ_Height[] = {
		46,46,46,46,46, 84,100,100,100,84, 170,170,170, 200,200};
	static final int MZ_PictX[] = {
		48,21,21,1,1, 209,101,36,1,209, 566,367,322, 259,289};
	static final int MZ_PictY[] = {
		153,1,1,1,153, 28,50,50,50,114, 16,16,16, 0,0};

	public static final int IMGTYPE_EMPTY = 0;
	public static final int IMGTYPE_ITEM = 1;
	public static final int IMGTYPE_WALL = 2;

    // NEW Mapping variables (% based) format [cell][render info]
    // cells = far fl-fr, med fl-fr, short l-r, pbl+pbr
    // render info = wallx,wallw,wallh,walltxtrstart,walltxtrend,
    //				sidex,sidew,sidesh,sideeh,sidetxtrstar,sidetxtrend
    private static final int RENDERDATA[][] =
    {
        {0,20,20,0,255, 20,10,20,10,0,255},	// Far Far-left
        {20,20,20,0,255, 40,8,20,4,0,255}, // Far Left
        {40,20,20,0,255, -1,0,0,0,0,0}, // Far Center
        {60,20,20,0,255, 52,8,4,20,0,255}, // Far Right
        {80,20,20,0,255, 70,10,10,20,0,255}, // Far Far-right
        
        {-1,-1,-1,-1,-1, 0,20,40,20,80,255}, // Medium Far-left
        {0,30,40,64,255, 30,10,40,20,0,255}, // Medium Left
        {30,40,40,0,255, -1,-1,-1,-1,-1,-1}, // Medium Center
        {70,30,40,0,192, 60,10,20,40,0,255}, // Medium Right
        {-1,-1,-1,-1,-1, 80,20,20,40,0,175}, // Medium Far-right

        {0,10,80,224,255, 10,20,80,40,0,255}, // Short Left
        {10,80,80,0,255, -1,-1,-1,-1,-1,-1}, // Short Center
        {90,10,80,0,32, 70,20,40,80,0,255}, // Short Right

        {-1,-1,-1,-1,-1, 0,10,100,80,240,255}, // Point-Blank Left
        {-1,-1,-1,-1,-1, 90,10,80,100,0,16}, // Point-Blank Right
    };
    private static final int WALL_X = 0;
    private static final int WALL_W = 1;
    private static final int WALL_H = 2;
    private static final int WALL_TXRSTART = 3;
    private static final int WALL_TXREND = 4;
    private static final int SIDE_X = 5;
    private static final int SIDE_W = 6;
    private static final int SIDE_SH = 7;
    private static final int SIDE_EH = 8;
    private static final int SIDE_TXRSTART = 9;
    private static final int SIDE_TXREND = 10;

    
// *********************
// ***** VARIABLES *****
// *********************
    
    int nImageType;

	Image iCells[] = new Image[15];

    protected int _strips[][];
    protected int _width;
    protected int _height;
    
    
	ImgCell(Image img, int nType, Component c)
    {
        nImageType = nType;
        prepareStrips(img, c);
	}

    // TEST
    public void test(Graphics g, ImageObserver o)
    {
        int wx, wy, ww, wh;
        int sx, ssy, sey, sw, ssh, seh;
        
        g.setColor(Color.green);
        for (int cntr = 0; cntr < 15; ++cntr)
        {
            wx = RENDERDATA[cntr][WALL_X] * 468 / 100;
            wh = RENDERDATA[cntr][WALL_H] * 360 / 100;
            ww = RENDERDATA[cntr][WALL_W] * 468 / 100;
            wy = (360 - wh) / 2;
            g.drawRect(wx,wy, ww,wh);

            sx = RENDERDATA[cntr][SIDE_X] * 468 / 100;
            sw = RENDERDATA[cntr][SIDE_W] * 468 / 100;
            ssh = RENDERDATA[cntr][SIDE_SH] * 360 / 100;
            ssy = (360 - ssh) / 2;
            seh = RENDERDATA[cntr][SIDE_EH] * 360 / 100;
            sey = (360 - seh) / 2;

            g.drawLine(sx,ssy, sx+sw, sey);
            g.drawLine(sx,ssy+ssh, sx+sw, sey+seh);
        }
    }

    
// *****************************************
// **** METHODS - new renderer methods *****
// *****************************************

	/**
	 * Prepare an image strip to be converted into a series of cells
	 */
    public void prepareStrips(Image img, Component c)
    {
        int w = img.getWidth(c);
        int h = img.getHeight(c);
        if ((w < 1) || (h < 1))
        	return;
        int raw[] = new int[w*h];
        PixelGrabber pg = new PixelGrabber(img, 0, 0, w, h, raw, 0, w);

        try
        {
            pg.grabPixels();
        }
        catch (InterruptedException e)
        {
            System.err.println("interrupted grabbing texture pixels!");
        }

        _strips = new int[w][h];
        for (int cntrW = 0; cntrW < w; ++cntrW)
        {
            for(int cntrH = 0; cntrH < h; ++cntrH)
            {
                _strips[cntrW][cntrH] = raw[cntrH*w+cntrW];
            }
        }
        _width = w;
        _height = h;        
    }
    
    /**
     * sets a memory image to blank values
     * 
     * @param raw array to be cleard
     * @param w width of array
     * @param h height of array
     * @param value value to place in array
     */
    public void clearOverlayRaw(int raw[], int w, int h, int value)
    {
        int size = w * h;
        for (int cntr = 0; cntr < size; ++cntr)
        {
            raw[cntr] = value;
        }
    }

	/**
	 * fills memory image with background image
	 * 
	 * @param raw memory image array to fill 
	 * @param w width of array
	 * @param h height of array
	 * @param img image to use as backdrop
	 */
    public void setBackdrop(int raw[], int w, int h, Image img)
    {
        PixelGrabber pg = new PixelGrabber(img, 0, 0, w, h, raw, 0, w);
        try
        {
            pg.grabPixels();
        }
        catch (InterruptedException e)
        {
            System.err.println("interrupted grabbing texture pixels!");
        }
        
    }
    
    /**
     * Renders memory image strip
     * @param raw
     * @param w
     * @param h
     * @param logpos
     */
    public void renderOverlayRaw(int raw[], int w, int h, int logpos)
    {
        if (nImageType == IMGTYPE_EMPTY)
            return;
        
        int wx, wy, ww, wh;
        int sx, ssy, sey, sw, ssh, seh;
        int cntrX, cntrY, offset;
        int pixel = (255<<24) | 255;
        int slice[], txDif, txStart;
        
        wx = RENDERDATA[logpos][WALL_X] * w / 100;
        wh = RENDERDATA[logpos][WALL_H] * h / 100;
        ww = RENDERDATA[logpos][WALL_W] * w / 100;
        wy = (h - wh) / 2;
        sx = RENDERDATA[logpos][SIDE_X] * w / 100;
        sw = RENDERDATA[logpos][SIDE_W] * w / 100;
        ssh = RENDERDATA[logpos][SIDE_SH] * h / 100;
        ssy = (h - ssh) / 2;
        seh = RENDERDATA[logpos][SIDE_EH] * h / 100;
        sey = (h - seh) / 2;

        if (nImageType == IMGTYPE_WALL)
        {
            // draw the wall
            if (ww > 0)
            {
                txStart = RENDERDATA[logpos][WALL_TXRSTART];
                txDif = RENDERDATA[logpos][WALL_TXREND] - txStart;
                for (cntrX = wx; cntrX < (wx+ww); ++cntrX)
                {
                    slice = _strips[(txStart + (cntrX-wx)*txDif/ww)*_width/256];
                    offset = wy * w + cntrX;
                    for (cntrY = 0; cntrY < wh; ++cntrY)
                    {
                        raw[offset] = slice[cntrY * slice.length / wh];
                        offset += w;
                    }
                }
            }
            
            // draw the side wall
            if (sw > 0)
            {
                int dif = seh - ssh;
                int colH;
                txStart = RENDERDATA[logpos][SIDE_TXRSTART];
                txDif = RENDERDATA[logpos][SIDE_TXREND] - txStart;
                for (cntrX = sx; cntrX < (sx+sw); ++cntrX)
                {
                    colH = dif * (cntrX - sx) / sw + ssh;
                    slice = _strips[(txStart + (cntrX-sx)*txDif/sw)*_width/256];
                    
                    offset = (h-colH)/2 * w + cntrX;
                    for (cntrY = 0; cntrY < colH; ++cntrY)
                    {
                        raw[offset] = slice[cntrY * slice.length / colH];
                        offset += w;
                    }
                }
            }
            
        }
        else if ( (nImageType == IMGTYPE_ITEM) && (ww > 0) )
        {
            int sprX, sprY, sprW, sprStart;

            if (logpos < 5)
            {
                sprX = wx + (ww - _width/4) / 2;
                sprY = wy + wh * 8 / 10 - _height/4;
                sprStart = Math.max(sprX * -1, 0);
                sprW = Math.min((w - sprX), _width / 4);
                for (cntrX = sprStart; cntrX < sprW; ++cntrX)
                {
                    slice = _strips[cntrX * 4];
                    offset = sprY * w + sprX + cntrX;
                    for (cntrY = 0; cntrY < (_height/4); ++cntrY)
                    {
                        if (slice[cntrY*4] != 0)
                            raw[offset] = slice[cntrY*4];
                        offset += w;
                    }
                }
            }
            else if (logpos < 10)
            {
                sprX = wx + (ww - _width/2) / 2;
                sprStart = Math.max(sprX * -1, 0);
                sprY = wy + wh * 8 / 10 - _height/2;
                sprW = Math.min((w - sprX), _width / 2);
                for (cntrX = sprStart; cntrX < sprW; ++cntrX)
                {
                    slice = _strips[cntrX * 2];
                    offset = sprY * w + sprX + cntrX;
                    for (cntrY = 0; cntrY < (_height/2); ++cntrY)
                    {
                        if (slice[cntrY*2] != 0)
                            raw[offset] = slice[cntrY*2];
                        offset += w;
                    }
                }
            }
            else if (logpos < 13)
            {
                sprX = wx + (ww - _width) / 2;
                sprStart = Math.max(sprX * -1, 0);
                sprY = wy + wh * 8 / 10 - _height;
                sprW = Math.min(w - sprX, _width);
                for (cntrX = sprStart; cntrX < sprW; ++cntrX)
                {
                    slice = _strips[cntrX];
                    offset = sprY * w + sprX + cntrX;
                    for (cntrY = 0; cntrY < (_height); ++cntrY)
                    {
                        if (slice[cntrY] != 0)
                            raw[offset] = slice[cntrY];
                        offset += w;
                    }
                }
            }
        }
    }
}
